----------------------------------------------------------------------------------------------------------------
--- DEM Digital Elevation Map importer Scripted FREE Plugin OBJECT  for  3DsVIZr3.x&3DsMAXr3.x
---	Import any binary dem file and convert into WGS84 real world coords,
---	NEED the BinaryData.dlx 1.04 Plugin script extension from J. Marcin Iwanski http://members.xoom.com/Glus
--- put files with extension (.dem) added in same folder with ESRI like Headers (.hdr)
--- Download the DEM files at http://www.ngdc.noaa.gov/seg/topo/gltiles.shtml
--- Download headers at ftp://ftp.ngdc.noaa.gov/GLOBE_DEM/data/lat/esri/hdr/
--- May import other Binary 16 bits files both in Intel or Motorolla byte order (GTOPO30 dem)
--- Put on max3-viz3/scripts/startup folder and restart max. Find it on Create Objects, ARCHIDATA-sa category
--- if you need increase script memory : uncomment the following heapsize+=100000000 command (remove the ---)
--- by Stephen Leger 				contact: steph@secondworld.com	---
--- http://www.secondworld.com   	http://www.archidata-sa.com		---
-----------------------------------------------------------------------
--- heapsize+=100000000 

plugin SimpleObject DEM_Mesh 
category:"ARCHIDATA-sa"
name:"DEM_Mesh"
classID:#(0x34913168,0x6eb63dd)
(	local FlipByte
	local RWGS84=6378.137
	fn readhdr f=
	(	str = readLine f
		ordstr= filterstring str " "
		return ordstr[2]
		)
	fn flipbyteorder str=
		(	local wrkstr
			if FlipByte then
			(	wrkstr=for n in 9 to 16 collect bit.get str n
				wrkstr+=for n in 1 to 8 collect bit.get str n
				for n in 1 to wrkstr.count do str=bit.set str n wrkstr[n]
				str = signit str 16	
				return str
				)
			else 
			(	str = signit str 16
				return str
				)
			)---end fn

	local Ellipsoid_names=#("User defined",\
				"AA/Airy 1830",\
				"AA/Airy 1830 modified",\
				"AN/Australian nationnal",\
				"BR/Bessel 1841",\
				"BN/Namibia",\
				"CC/Clarke 1866",\
				"CD/Clarke 1880 (DMA)",\
				"EB/Everest",\
				"EA/India 1830",\
				"EC/India 1956",\
				"EE/W.Malaysia 1948",\
				"ED/W.Malaysia 1969",\
				"RF/Geodetic ref. 1980",\
				"HE/Helmert 1906",\
				"HO/Hough 1960",\
				"IN/Internationnal 1924",\
				"KA/Krassovsky 1940",\
				"FA/Modified Fischer 1960",\
				"SA/South American 1969",\
				"WD/WGS 72",\
				"WE/WGS84",\
				"Simple Sphere"\
				)
					
	local Ellipsoid_params=#(#(0,298.257225636),\	---Delta a (WGS84 based), f
				#(573.604,299.3249646),\
				#(573.604,298.25),\
				#(-23,299.1528128),\
				#(739.845,299.1528128),\
				#(653.135,299.1528128),\
				#(-69.4,294.9786982),\
				#(-112.145,293.465),\
				#(838.444,300.8017),\
				#(860.655,300.8017),\
				#(835.757,300.8017),\
				#(832.937,300.8017),\
				#(841.336,300.8017),\
				#(0,298.257222101),\
				#(-63,298.3),\
				#(-133,297),\
				#(-251,297),\
				#(-108,298.3),\
				#(-18,298.3),\
				#(-23,298.25),\
				#(2,298.26),\
				#(0,298.257223536),\
				#(0,1.30624e+007)\
				)
	local Datum_names=#("User defined",\
					"WGS84",\
					"WGS72",\
					"ED50",\
					"ED79",\
					"ED87",\
					"Austria NS",\
					"Belgium 50",\
					"Berne 1873",\
					"CH-1903",\
					"Danish GI 1934",\
					"Nouv Trig France Greenwitch",\
					"Nouv Trig France Paris",\
					"Potsdam",\
					"GGRS87",\
					"Hjorsey 55",\
					"Ireland 65",\
					"Italy 1940",\
					"Nouv Trig de Lux",\
					"Netherlands 1921",\
					"OSGB 36",\
					"Portugal DLX",\
					"Portugal 1973",\
					"RNB 73",\
					"RT 90",\
					"NAD 27",\
					"Nad 83"\
					)
	local Datum_corrections=#(	#(0,0,0,0,0,0,0),\  ---Tx,Ty,Tz,Rx,Ry,Rz,scale
					#(0,0,0,0,0,0,0),\
					#(0,0,-4.5,0,0,0.554,-0.22),\
					#(87,98,121,0,0,0,0),\
					#(86,98,119,0,0,0,0),\
					#(82.5,91.7,117.7,-0.1338,0.0625,0.047,-0.045),\
					#(-595.6,-87.3,-473.3,-4.7884,-0.0672,-5.785,-2.555),\
					#(55,49,158,0,0,0,0),\
					#(-649,-9,-376,0,0,0,0),\
					#(-660.1,-13.1,-369.2,-0.8048,-0.5777,-0.9522,-5.660),\
					#(-662,-18,-734,0,0,0,0),\
					#(168,60,-320,0,0,0,0),\
					#(168,60,-320,0,0,-8414.03,0),\
					#(-587,-16,-393,0,0,0,0),\
					#(-199.6,75.1,246.3,-0.0202,-0.0034,-0.0135,0.015),\
					#(73,-46,86,0,0,0,0),\
					#(-506,122,-611,0,0,0,0),\
					#(133,50,-97,0,0,-44828.4,0),\
					#(262,-75,-25,0,0,0,0),\
					#(-719,-47,-640,0,0,0,0),\
					#(-375,111,-431,0,0,0,0),\
					#(-504.1,220.9,-563.0,0,0,0.554,-0.220),\
					#(227,-97.5,-35.4,0,0,0.554,-0.220),\
					#(104,-80,75,0,0,0,0),\
					#(-424.3,80.5,-613.1,-4.3965,1.9866,-5.1846,0),\
					#(8,-160,-176,0,0,0,0),\
					#(0,0,0,0,0,0,0)\
					)
	

	parameters  infoblock rollout:roInfo
	(	lat	type:#float animatable:true	default:0
		lon	type:#float animatable:true	default:0
		size_x	type:#float animatable:true	default:1
		size_y	type:#float animatable:true	default:1
		)	

	parameters	paramblock rollout:roMesh
	(	BaseFileName type:#string
		Flag_first	type:#Boolean default:false 	
		a 	 	type:#float ui:spnmaj default:6378.137
		b  		type:#float ui:spnmin default:6356.7523141
		ePreset type:#integer animatable:false default:22
		dPreset type:#integer animatable:false default:2
		step_y 	type:#integer animatable:true ui:spnstep_y default:1
		step_x 	type:#integer animatable:true ui:spnstep_x default:1
		Startv 	type:#integer animatable:true ui:spnStartv default:1
		Starth 	type:#integer animatable:true ui:spnStarth default:1
		countv 	type:#integer animatable:true ui:spncountv default:1
		counth 	type:#integer animatable:true ui:spncounth default:1
		Rx type:#angle ui:spnrx default:0
		Ry type:#angle ui:spnry default:0
		Rz type:#angle ui:spnrz default:0
		Dx type:#angle ui:spndx default:0
		Dy type:#angle ui:spndy default:0
		Dz type:#angle ui:spndz default:0
		sc type:#float ui:spnsc default:0
		)
	parameters	paramalt rollout:roMaterial
	(	alt1 	type:#float animatable:true ui:spnAlt1 default:50
		alt2 	type:#float animatable:true ui:spnAlt2 default:200
		alt3 	type:#float animatable:true ui:spnAlt3 default:750
		alt4 	type:#float animatable:true ui:spnAlt4 default:1900
		alt5 	type:#float animatable:true ui:spnAlt5 default:3000
		)
	rollout roInfo "Info"
	(	Group "File"
		(	label lab102 "Name :"
			label lab101 "Points:"
			)
		group "Coords"
		( 	label lab201 "Long : "
			label lab203 "Latt : "
			label lab205 "Center : "
			)	
		on roInfo open do
		(	if BaseFileName != undefined do
			(	lab101.text= ("Points : "+(counth/step_x*countv/step_y)as string+" / "+(counth*countv)as string)
				lab102.text= ("Name : "+getFilenameFile BaseFileName+".dem")
				lab201.text= ("Long : "+lon as string+" / "+(lon+grid_size_x*counth)as string)
				lab203.text= ("Latt : "+lat as string+" / "+(lat-grid_size_y*countv)as string)
				lab205.text= ("Center : "+latC as string+" / "+lonC as string)
				)
			)
		)	
		
	rollout roMesh "Mesh"
	(	group "Mesh Density"
		(	spinner spnstep_x "Step X :" range:[1,10000,20] type:#integer enabled:false
			spinner spnstep_y "Step Y :" range:[1,10000,20] type:#integer enabled:false
			)
		group "Clipping"
		(	spinner spnStartv "Top :" range:[1,100000,1] type:#integer enabled:false
			spinner spncountv "Bottom :" range:[1,100000,1] type:#integer enabled:false
			spinner spnStarth "Left :" range:[1,100000,1] type:#integer enabled:false
			spinner spncounth "Right :" range:[1,100000,1] type:#integer enabled:false
			)
		group "Ellipsoid"
		(	dropdownlist el_preset	items:Ellipsoid_names selection:22 align:#right  width:140
			spinner spnmaj "semimajor axis:" type:#float range:[-1e32,1e32,6378.137] enabled:false
			spinner spnmin "semiminor axis:" type:#float range:[-1e32,1e32,6356.7523141] enabled:false
			) 
			
		group "DATUM transform -> WGS84"
		(	dropdownlist dr_preset	items:Datum_names selection:2 align:#right  width:140
			label labTrans "Rotation  Displacement" enabled:false
			spinner spnrx "x" type:#float range:[-1e32,1e32,0] enabled:false fieldwidth:55 across:2
			spinner spndx "" type:#float range:[-1e32,1e32,0] enabled:false  fieldwidth:55
			spinner spnry "y" type:#float range:[-1e32,1e32,0] enabled:false fieldwidth:55 across:2
			spinner spndy "" type:#float range:[-1e32,1e32,0] enabled:false fieldwidth:55
			spinner spnrz "z" type:#float range:[-1e32,1e32,0] enabled:false fieldwidth:55 across:2
			spinner spndz "" type:#float range:[-1e32,1e32,0] enabled:false fieldwidth:55
			spinner spnsc "Scale :" type:#float range:[-1e32,1e32,0] enabled:false
			)

		on el_preset selected i do
		(	spnmaj.enabled=spnmin.enabled= i==1
			if i > 1 then	
			(	a= RWGS84-(Ellipsoid_params[i][1]/1000.0)
				b= a*(1-(1/Ellipsoid_params[i][2] as float))
				Ellipsoid_names[1]="User defined"
				el_preset.items=Ellipsoid_names
				)
			else if ePreset !=1 do el_preset.selected=("Based on :"+el_preset.items[ePreset])
			ePreset=i
			redrawviews()
			)

		on dr_preset selected i do
		(	spnsc.enabled=spndz.enabled=spnrz.enabled=spndy.enabled=spnry.enabled=spndx.enabled=spnrx.enabled=labTrans.enabled= i==1 
			if i > 1 then	
			(	Dx=Datum_corrections[i][1]
				Dy=Datum_corrections[i][2]
				Dz=Datum_corrections[i][3]
				Rx=Datum_corrections[i][4]
				Ry=Datum_corrections[i][5]
				Rz=Datum_corrections[i][6]
				sc=Datum_corrections[i][7]
				Datum_names[1]="User defined"
				dr_preset.items=Datum_names
				)
			else if dPreset !=1 do	dr_preset.selected=("Based on :"+dr_preset.items[dPreset])
			dPreset=i
			redrawviews()
			)

		on roMesh open do 
		(	dr_preset.selection = dPreset
			el_preset.selection = ePreset
			if BaseFileName != undefined do
			(	spnstep_y.enabled=true
				spnstep_x.enabled=true
				spnStartv.enabled=true
				spnStarth.enabled=true
				spncountv.enabled=true
				spncounth.enabled=true
				)
			)

		)


	rollout roMaterial "Material"
	(	group "Altitude material"
		(	spinner spnAlt1 "Alt 1 :" range:[1,10000,50] type:#float 
			spinner spnAlt2 "Alt 2 :" range:[1,10000,200] type:#float
			spinner spnAlt3 "Alt 3 :" range:[1,10000,750] type:#float 
			spinner spnAlt4 "Alt 4 :" range:[1,10000,1900] type:#float
			spinner spnAlt5 "Alt 5 :" range:[1,10000,3000] type:#float
			)
		)


	rollout roAbout "About"
	(	group "Info"
		(	label lab13 "DEM ellipsoidal importer"
			label lab18 "version Beta 02.03"
			)
		group "Contact"
		(	label lab07 "steph@secondworld.com"	
			label lab02 "http://www.secondworld.com"
			label lab09 "http://www.archidata-sa.com"
			)
		)
----------------------------------------
	
	on buildmesh do
	(			
		if BaseFileName == undefined do
		(	BaseFileName = (getOpenFileName "Choose DEM File" types:"Header (*.hdr)|*.hdr|Dem (*.dem)|*.dem|All files(*.*)|*.*|") as string
			roInfo.lab102.text= ("Name : "+getFilenameFile BaseFileName+".dem")	
			flag_first=on
			)

		start = timeStamp() 

		If M_Progress != undefined then CloseRolloutFloater M_Progress
		M_Progress = NewRolloutFloater "Importing dem" 350 100 100 100
		Rollout ProgR ""
		(	Label Progt "" Align:#Left OffSet:[0,0]
			ProgressBar ProgB width:300 Offset:[0,0] Height:15
			)
		AddRollout ProgR M_Progress 
------------------------------------
	
		ProgR.Progt.Text = "Reading header"
		
		strName =	( getFilenamePath BaseFileName+getFilenameFile BaseFileName)
		HDRIN = strName+".hdr"
		DEMIN = strName+".dem"

		h = openFile 	HDRIN
			if h == undefined then 
			(	return false	
				h = openFile HDRIN
				)		
		FlipByte= (readhdr h == "M")  ---check byte order M=Motorolla I=Intel
		skiptonextline h
		profils = readhdr h as integer
		points = readhdr h as integer
		for n in 1 to 5 do skiptonextline h
		miss_flag = readhdr h as integer
		left_map_x = readhdr h as float
		upper_map_y = readhdr h as float
		grid_size_x = readhdr h as float
		grid_size_y = readhdr h as float
		
		close h
----------------------------
		
		if flag_first do    
		(	countv=profils
			counth=points
			step_x= (points-mod points 50)/50 
			step_y= (profils-mod profils 50)/50
			roMesh.spnstep_y.enabled=true
			roMesh.spnstep_x.enabled=true
			roMesh.spnStartv.enabled=true
			roMesh.spnStarth.enabled=true
			roMesh.spncountv.enabled=true
			roMesh.spncounth.enabled=true
			)
		flag_first=off	
--------------------------------

		while starth+counth > (points+1)  do counth -=1
		while startv+countv > (profils+1) do countv -=1

		counth = counth- (mod counth step_x)
		countv = countv- (mod countv step_y)	

		lonC = left_map_x +(Starth-1+counth/2)*grid_size_x
		latC = upper_map_y-(Startv-1+countv/2)*grid_size_y
		lat  = upper_map_y-grid_size_y*(startv-1.)

		center=[0,0,0]	

		flat=(a-b)/a	---f= flattening of ellipsoid
		e2=flat*(2-flat)---e= eccentricity of ellipsoid
	
		roInfo.lab101.text= ("Points : "+(counth/step_x*countv/step_y)as string+" / "+(counth*countv)as string)
		roInfo.lab201.text= ("Long : "+lon as string+" / "+(lon+grid_size_x*counth)as string)
		roInfo.lab203.text= ("Latt : "+lat as string+" / "+(lat-grid_size_y*countv)as string)
		roInfo.lab205.text= ("Center : "+latC as string+" / "+lonC as string)
		

---		try
---		(	
			vert=#()
			mid=#()
			f= BinaryFile DEMIN
			seek f (2*startv*points) 0
			seek f (2*starth) 1

			ProgR.Progt.Text = "Reading vertices                   "
		
			for b in 1 to countv by step_y do
			(	lon = left_map_x+grid_size_x*(starth-1.)
				for g in 1 to counth by step_x do 
				(	val=flipbyteorder (ReadBinWord f)	
					mat=(	if val == miss_flag then 1
							else if val < alt1 then 2
							else if val < alt2 then 3
							else if val < alt3 then 4
							else if val < alt4 then 5
							else if val < alt5 then 6
							else 7
							)
					append mid mat
					append mid mat	
					val = if val != miss_flag then val/1000. else 0
					v= a*sqrt(1-e2 *(sin lat)^2)
					pos = [(v+val)*(cos lat)*(cos (lon-90)),(v+val)*(cos lat)*(sin (lon-90)),(v*(1-e2)+val)*sin lat]

					append vert pos

					lon += grid_size_x*step_x
					seek f ((step_x-1)*2) 1	
					)
				for n in 1 to 2 do deleteitem mid mid.count
				seek f (2*(points-counth)) 1	
				seek f (2*(step_y-1)*points)	1  --- skip nlignes
				lat -= grid_size_y*step_y
				ProgR.ProgB.value = (b/countv as float *100)
				ProgR.ProgB.color = (if ProgR.ProgB.value < 30 then red else if ProgR.ProgB.value < 60 then yellow else green)
				)
			close f	
------------------------

			ProgR.Progt.Text = "Computing faces             "
	
			pts=(counth/step_x)
			for n in 1 to ((pts-1)*2) do deleteitem mid mid.count
		
			fc=#()
			for b in 1 to (countv/step_y-1) do
				(	for g in 1 to (pts-1) do
					(	append fc [(b-1)*pts+g+1,(b-1)*pts+g,b*pts+g]
						append fc [b*pts+g+1,(b-1)*pts+g+1,b*pts+g]	
						)
					ProgR.ProgB.value = (b/(countv/step_y-1) as float *100)
					ProgR.ProgB.color = (if ProgR.ProgB.value < 30 then red else if ProgR.ProgB.value < 60 then yellow else green)
					)
-------------------------
				
				ProgR.Progt.Text = "Building Mesh               "
			
				m= mesh
				setMesh m vertices:vert faces:fc materialIds:mid	 --- fill mesh with collected datas
				update m
---				)
---			catch 
---			(	messagebox ("Import failure file :\n"+getFileNameFile BaseFileName+".dem")
---				)
			end = timeStamp() 
			Tcount= ((end - start) / 1000.0)
			print ("Time : "+Tcount as string+" seconds")
			If M_Progress != undefined then CloseRolloutFloater M_Progress
			)
---------------------------

	tool create
	(	on mousePoint click do
			case click of 
			(   1: ()
            	2: #stop
       			)        
		on mouseMove click do 
			case click of 
			(	2:()
        		)        
		)

	)
