-----------------------------------------------------------------------
---	Ellipsoidal_proj FREE Script Modifier for 3DsVIZr3.x&3DsMAXr3.x	---
--- Universal transverse Mercator - Lambert Conical - Sphere 		---
--- Ellipsoidal projections of Earth WGS84 model					---
--- System unit : km												---
--- by Stephen Leger 				contact: steph@secondworld.com	---
--- http://www.secondworld.com   	http://www.archidata-sa.com		---
-----------------------------------------------------------------------
plugin simpleMod Ellipsoidal_proj --- 20 minutes/ 1 000 000 pts P3 800mhz
category:"ARCHIDATA-sa"
name:"Ellipsoidal_proj"
classID:#(0x380266de, 0x72a92068)
(	local RWGS84=6378.137
	local limit
	local cot
	fn cot x = 1/tan x		--- cotangent
	fn limit x base lim=
	(	local val=if x > base+lim then base+lim else if x < base-lim then base-lim else x
		return val
		)				  
	local Ellipsoid_names=#("User defined",\
				"AA/Airy 1830",\
				"AA/Airy 1830 modified",\
				"AN/Australian nationnal",\
				"BR/Bessel 1841",\
				"BN/Namibia",\
				"CC/Clarke 1866",\
				"CD/Clarke 1880 (DMA)",\
				"EB/Everest",\
				"EA/India 1830",\
				"EC/India 1956",\
				"EE/W.Malaysia 1948",\
				"ED/W.Malaysia 1969",\
				"RF/GRS 1980",\
				"HE/Helmert 1906",\
				"HO/Hough 1960",\
				"IN/Internationnal 1924",\
				"KA/Krassovsky 1940",\
				"FA/Modified Fischer 1960",\
				"SA/South American 1969",\
				"WD/WGS 72",\
				"WE/WGS84",\
				"Simple Sphere"\
				)
	local Ellipsoid_params=#(#(0,298.257225636),\	---Delta a (WGS84 based), f
				#(573.604,299.3249646),\
				#(573.604,298.25),\
				#(-23,299.1528128),\
				#(739.845,299.1528128),\
				#(653.135,299.1528128),\
				#(-69.4,294.9786982),\
				#(-112.145,293.465),\
				#(838.444,300.8017),\
				#(860.655,300.8017),\
				#(835.757,300.8017),\
				#(832.937,300.8017),\
				#(841.336,300.8017),\
				#(0,298.257222101),\
				#(-63,298.3),\
				#(-133,297),\
				#(-251,297),\
				#(-108,298.3),\
				#(-18,298.3),\
				#(-23,298.25),\
				#(2,298.26),\
				#(0,298.257223536),\
				#(0,1.30624e+007)\
				)
	local Datum_names=#("User defined",\
					"WGS84",\
					"WGS72",\
					"ED50",\
					"ED79",\
					"ED87",\
					"Austria NS",\
					"Belgium 50",\
					"Berne 1873",\
					"CH-1903",\
					"Danish GI 1934",\
					"Nouv Trig France Greenwitch",\
					"Nouv Trig France Paris",\
					"Potsdam",\
					"GGRS87",\
					"Hjorsey 55",\
					"Ireland 65",\
					"Italy 1940",\
					"Nouv Trig de Lux",\
					"Netherlands 1921",\
					"OSGB 36",\
					"Portugal DLX",\
					"Portugal 1973",\
					"RNB 73",\
					"RT 90",\
					"NAD 27",\
					"Nad 83"\
					)
	local Datum_corrections=#(	#(0,0,0,0,0,0,0),\  ---Tx,Ty,Tz,Rx,Ry,Rz,scale
					#(0,0,0,0,0,0,0),\
					#(0,0,4.5,0,0,-0.554,0.22),\
					#(-87,-98,-121,0,0,0,0),\
					#(-86,-98,-119,0,0,0,0),\
					#(-82.5,-91.7,-117.7,0.1338,-0.0625,-0.047,0.045),\
					#(595.6,87.3,473.3,4.7884,0.0672,5.785,2.555),\
					#(-55,49,-158,0,0,0,0),\
					#(649,9,376,0,0,0,0),\
					#(660.1,13.1,369.2,0.8048,0.5777,0.9522,5.660),\
					#(662,18,734,0,0,0,0),\
					#(-168,-60,320,0,0,0,0),\
					#(-168,-60,320,0,0,8414.03,0),\
					#(587,16,393,0,0,0,0),\
					#(199.6,-75.1,-246.3,0.0202,0.0034,0.0135,-0.015),\
					#(-73,46,-86,0,0,0,0),\
					#(506,-122,611,0,0,0,0),\
					#(-133,-50,97,0,0,44828.4,0),\
					#(-262,75,25,0,0,0,0),\
					#(719,47,640,0,0,0,0),\
					#(375,-111,431,0,0,0,0),\
					#(504.1,-220.9,563.0,0,0,-0.554,0.220),\
					#(-227,97.5,35.4,0,0,-0.554,0.220),\
					#(-104,80,-75,0,0,0,0),\
					#(424.3,-80.5,613.1,4.3965,-1.9866,5.1846,0),\
					#(-8,160,176,0,0,0,0),\
					#(0,0,0,0,0,0,0)\
					)
	local Proj_names=#(	"Transverse Mercator UTM",\
					"Lambert conical, one standard parallel",\
					"Lambert conical, two standard parallel",\
					"Sphere"\
					)
	parameters main rollout:params
	(	Derive type:#angle default:(-3)
		Rx type:#angle ui:spnrx default:0
		Ry type:#angle ui:spnry default:0
		Rz type:#angle ui:spnrz default:0
		Dx type:#angle ui:spndx default:0
		Dy type:#angle ui:spndy default:0
		Dz type:#angle ui:spndz default:0
		sc type:#float ui:spnsc default:0
		Quant type:#float ui:spnqnt default:100
		No	type:#float ui:spnno default:0		---Northing of true origin
		Eo	type:#float ui:spneo default:0		---Easting of true origin
		Fo	type:#float ui:spnfo default:1		---scale factor on central meridian
		lat0 	type:#angle ui:spnelo default:0 		---lattitude of true origin
		lon0 	type:#angle ui:spndero default:(-3)		---Longitude of true origin
		lat1 	type:#angle ui:spnelo1 default:10	--- parallel1
		lat2 	type:#angle ui:spnelo2 default:40	--- parallel2
		a  	type:#float ui:spnmaj default:6378.137
		b  	type:#float ui:spnmin default:6356.7523141
		zone   type:#integer ui:spnzone animatable:false default:30
		Centerext type:#boolean animatable:false default:true
		Auto_Update type:#boolean animatable:false default:true
		pPreset type:#integer animatable:false default:1
		ePreset type:#integer animatable:false default:22
		dPreset type:#integer animatable:false default:2
		)
	rollout params "Parameters"
	(	group "Ellipsoid"
		(	dropdownlist el_preset	items:Ellipsoid_names selection:22 align:#right  width:140
			spinner spnmaj "semimajor axis:" type:#float range:[-1e32,1e32,6378.137] enabled:false
			spinner spnmin "semiminor axis:" type:#float range:[-1e32,1e32,6356.7523141] enabled:false
			) 
		group "DATUM transformation"
		(	dropdownlist dr_preset	items:Datum_names selection:2 align:#right  width:140
			label labTrans "Rotation      Displacement" enabled:false
			spinner spnrx "x" type:#float range:[-1e32,1e32,0] enabled:false fieldwidth:55 across:2
			spinner spndx "" type:#float range:[-1e32,1e32,0] enabled:false  fieldwidth:55
			spinner spnry "y" type:#float range:[-1e32,1e32,0] enabled:false fieldwidth:55 across:2
			spinner spndy "" type:#float range:[-1e32,1e32,0] enabled:false fieldwidth:55
			spinner spnrz "z" type:#float range:[-1e32,1e32,0] enabled:false fieldwidth:55 across:2
			spinner spndz "" type:#float range:[-1e32,1e32,0] enabled:false fieldwidth:55
			spinner spnsc "Scale :" type:#float range:[-1e32,1e32,0] enabled:false
			)
		group "Projection"
		(	dropdownlist pr_preset	items:Proj_names selection:1 align:#right  width:140
			spinner spnzone "UTM zone n" type:#integer range:[1,60,30] enabled:true 
			spinner spnelo  "ref Latt :" type:#float range:[-180,180,0] enabled:true
			spinner spndero "ref Long :" type:#float range:[-180,180,0] enabled:true
			spinner spnelo1  "Latt1 :" type:#float range:[-180,180,10] enabled:false
			spinner spnelo2  "Latt2 :" type:#float range:[-180,180,40] enabled:false
			spinner spnqnt "Effect amount :" type:#float range:[0,100,100] 			
			checkbox chkcenter "Projection center at [0,0,0]" checked:true 
			checkbox chkupdate "Auto Update"  checked:true
			)
		group "Local system Origin"
		(	spinner spnno 	"Northing :" type:#float range:[-1e32,1e32,0] enabled:true
			spinner spneo 	"Esting   :" type:#float range:[-1e32,1e32,0] enabled:true
			spinner spnfo 	"Meridian Scale:" type:#float range:[-1e32,1e32,0] enabled:true
			)	
	on chkcenter changed state do
	(	Centerext= state
		redrawviews()
		)
	on chkupdate changed state do
	(	Auto_update= state
		redrawviews()
		)
	on el_preset selected i do
	(	spnmaj.enabled=spnmin.enabled= i==1
		if i > 1 then	
		(	a= RWGS84-(Ellipsoid_params[i][1]/1000.0)
			b= a*(1-(1/Ellipsoid_params[i][2] as float))
			Ellipsoid_names[1]="User defined"
			el_preset.items=Ellipsoid_names
			)
		else
		(	if ePreset !=1 do
			(	el_preset.selected=("Based on :"+el_preset.items[ePreset])
				)
			)
		ePreset=i
		redrawviews()
		)
	on spnzone changed val do
	(	lon0 = (val-30)*6-3
		redrawviews()
		)
	on pr_preset selected i do
	(	pPreset=i
		spnzone.enabled= i== 1
		spnelo1.enabled=spnelo2.enabled= i==3
		if i==1 do lon0= (zone-30)*6-3
		redrawviews()		
		)
	
	on dr_preset selected i do
	(	spnsc.enabled=spndz.enabled=spnrz.enabled=spndy.enabled=spnry.enabled=spndx.enabled=spnrx.enabled=labTrans.enabled= i==1 
		if i > 1 then	
		(	Dx=Datum_corrections[i][1]
			Dy=Datum_corrections[i][2]
			Dz=Datum_corrections[i][3]
			Rx=Datum_corrections[i][4]
			Ry=Datum_corrections[i][5]
			Rz=Datum_corrections[i][6]
			sc=Datum_corrections[i][7]
			Datum_names[1]="User defined"
			dr_preset.items=Datum_names
			)
		else
		(	if dPreset !=1 do
			(	dr_preset.selected=("Based on :"+dr_preset.items[dPreset])
				)
			)
		dPreset=i
		redrawviews()
		)
		on params open do
		(	dr_preset.selection = dPreset
			el_preset.selection = ePreset
			pr_preset.selection = pPreset
			)
		)
	
	rollout infos "About"
	(	group "Info"
		(	label lab13 "Ellipsoidal Projections"
			label lab18 "version Beta 05.21"
			)
		group "Contact"
		(	label lab07 "steph@secondworld.com"
			label lab08 "http://www.secondworld.com"
			label lab09 "http://www.archidata-sa.com"
			)
		)		
	on map i p do
	(	if Auto_Update then
		(	if centerext do	center=[0,0,0]
			pos=p*(1+sc/1000.0)*((eulerangles (Rx/3600.0)  (Ry/3600.0) (Rz/3600.0)) as matrix3) + [(Dx/1000.0),(Dy/1000.0),(Dz/1000.0)]
			c  = sqrt (a^2-b^2)			--- ellipse ratio -->Foyers communs		
			lon= (arbaxis pos as eulerangles).z --- longitude
			x1 = distance [center.x,center.y] [pos.x,pos.y]
			z1 = abs(pos.z-center.z)
			ah = ( sqrt((x1-c)^2+z1^2)-sqrt((x1+c)^2+z1^2) )/2 --- hyperbola a (semimajor axis)
			bh = sqrt(c^2-ah^2)				--- hyperbola b (semiminor axis)
			re2= a-ah						--- ellipse r1
			re1= a+ah						--- ellipse r2
			x2 = abs (((re2^2) as float -re1^2)/(4*c)) --- projection along normal of ellipse 
			z2 = abs (sqrt(re1^2-((c-x2)^2) as float)) --- projection along normal of ellipse 
			if z2 > 10000000 or z2 < -10000000 do z2 = z1
			lat = (if pos.z > center.z then 1 else -1)*(atan(a/b as float*(z2 as float/x2)))	---	angle tangentiel f(angle polaire) 
			alt= sqrt(x1^2+z1^2)-sqrt(x2^2+z2^2)

	pos= case pPreset of 
	(	1: (	lat=limit lat lat0 86
				lon=limit lon lon0 86
				e2=((a^2)-(b^2))/(a^2)
				n=(a-b)/(a+b)
				v= a*1000*Fo*pow(1-e2*(sin lat)^2) -0.5
				Phi=a*1000*Fo*(1-e2)*pow(1-e2*(sin lat)^2) -1.5
				N2=(v/Phi)-1
				n3=if n^3==0 then 0.000001 else n^3
				M=b*1000*Fo*((1+n+5./4*n^2+5./4*n3)*degtorad(lat-lat0)-(3*n+3*n^2+21./8*n3)*sin(lat-lat0)*cos(lat+lat0)+(15./8*n^2+15./8*n3)*sin(2.*(lat-lat0))*cos(2.*(lat+lat0))-25./24*n3*sin(3.*(lat-lat0))*cos (3.*(lat+lat0)))
				Ix=M+No
				II=v/2.*sin lat*cos lat
				III=v/24.*sin lat*cos lat^3*(5-tan lat^2+9*N2)
				IIIA=v/720.*sin lat*cos lat^5*(61-58 *tan lat^3+tan lat^4)
				IV=v* cos lat
				Va=v/6.*cos lat^3*((v/Phi)-tan lat^2)
				VI=v/120.*cos lat^5*(5-18.*tan lat^2+tan lat^4+14.*N2-58.*tan lat^2*N2)
				ang=degtorad(lon-lon0)
				Ns=Ix+II*ang^2+III*ang^4+IIIA*ang^6
				Es=Eo+IV*ang+Va*ang^3+VI*ang^5
				[Es/1000.,Ns/1000.,alt])
		2: (	lat=limit lat lat0 86
				lon=limit lon lon0 176
				e2=((a^2)-(b^2))/(a^2)
				e1=sqrt e2
				v0= a/pow(1-e2*sin lat0^2) .5
				r0= v0*cot lat0
				r= r0*pow( (tan (45-.5*lat)*pow ((1+e1*sin lat)/(1-e1*sin lat)) (.5*e1))/(tan (45-.5*lat0)*pow ((1+e1*sin lat0)/(1-e1*sin lat0)) (.5*e1)) ) (sin lat0)
				theta= (lon-lon0)*sin lat0
				rm= eulerangles 180 0 90 as quat
				[r0-r*cos theta,r*sin theta,alt]*rm+[Eo,No,0]	)
		3: (	lat=limit lat lat0 86
				lon=limit lon lon0 176
				e2=((a^2)-(b^2))/(a^2)
				e1=sqrt e2
				v1=a/pow (1-e2*sin lat1^2) .5
				v2=a/pow (1-e2*sin lat2^2) .5
				sinlat0= (log (v1*cos lat1)-log (v2*cos lat2))/(log (tan (45-.5*lat1)*pow ((1+e1*sin lat1)/(1-e1*sin lat1)) (.5*e1))-log (tan (45-.5*lat2)*pow((1+e1*sin lat2)/(1-e1*sin lat2)) (.5*e1) ))
				r= v2*cos lat2/sinlat0*pow((tan (45-.5*lat)*pow ((1+e1*sin lat)/(1-e1*sin lat)) (.5*e1))/(tan (45-.5*lat0)*pow ((1+e1*sinlat0)/(1-e1*sinlat0)) (.5*e1)) ) (sinlat0)
				r0=v2*cos lat2/sinlat0*pow((tan (45-.5*lat0)*pow ((1+e1*sinlat0)/(1-e1*sinlat0)) (.5*e1))/(tan (45-.5*lat2)*pow ((1+e1*sin lat2)/(1-e1*sin lat2)) (.5*e1)) ) (sinlat0)
				theta=(lon-lon0)*sinlat0
				rm= eulerangles 180 0 90 as quat
				[r0-r*cos theta,r*sin theta,alt]*rm+[Eo,No,0])
		4:	(	rm = eulerangles (lat0+90-lat) 0 (lon-lon0) as quat
				[0,0,a+alt]*rm	)
			)
		if Quant < 100 then
			(	vecteur = normalize (pos-p)
				scalaire =distance p pos
				p= p + vecteur*scalaire*Quant/100
				)
			else p=pos
			)
		else p=p  --- if not autoUpdate
		) 	---	end on map i p do 
	)		--- end plugin